function [mtx] = createFullHBondMatrix_from_parseHBond_allOutput (hBondNdxData, hBondMatrix, pdb)
% this function create a nxm matrix of h-bond existance where n and m are
% the number of residues or the number of atom (the atom version might be
% very time and resource consuming - not recommended)
%
% The input of the function comes from:
%
% parseHBond_allOutput.m
%
% e.g.
%
% baseFn = '/data/housemartin/knapp/matlabWorkspace/util/parsing/ascii/g5_h-bond/';
% [hBondMatrix, hBondNdxData, hBondNum, pdb, allAtomMtxGroup1, allAtomMtxGroup2, allResMtxGroup1, allResMtxGroup2] = createFullHBondMatrix_from_parseHBond_allOutput ([baseFn, 'test.hbm.xpm'], [baseFn, 'test.hbn.ndx'], [baseFn, 'hBondNum.xvg'], [baseFn, 'JM22.firstFrame.pdb'],  false);
%
% useCAs = true;
% [mtx] = createFullHBondMatrix_from_parseHBond_allOutput (hBondNdxData, hBondMatrix, pdb, useCAs)
%
% B. Knapp 2015-12-14

pdb.coords = nanometer2angstroem(pdb.coords);

hbBondPresence = mean(hBondMatrix.mtx, 2);

hbBondPresence = hbBondPresence(end:-1:1); % flip the dimensions to make it match

%     % all the same size:
%     hBondMatrix.headerInfo.yAxis % e.g. 0-2826
%     hBondNdxData.donors
%     hBondNdxData.acceptors
%     hbBondPresence

% if useCAs
pdbCA = selectAtoms(pdb, {'CA'});
mySize = length(pdbCA.atomNumber); % the matrix where everything is saved on a per atom basis
pdbCA.residueNumber = 1:length(pdbCA.residueNumber);
[pdb] = changeResidueIndicesToConsecutiveNumbering (pdb, pdbCA);
% else
%     mySize = length(pdb.atomNumber); % the matrix where everything is saved on a per resiude basis
% end
mtx = zeros (mySize, mySize);

idxDonorH = 2;
idxDonorHeavy = 1;
idxShiftGromacs = 0;


for hBondIter = 1:size(hBondNdxData.donors,1)
    
    % gromacs ndx indexing  starts with 1
    % matlab array indexing starts with 1
    % vmd selection indexing starts with 0
    
    donorAtomNrH  = pdb.atomNumber(hBondNdxData.donors(hBondIter,idxDonorH)+idxShiftGromacs);
    donorAtomNrHeavy = pdb.atomNumber(hBondNdxData.donors(hBondIter,idxDonorHeavy)+idxShiftGromacs);
    acceptorAtomNr =  pdb.atomNumber(hBondNdxData.acceptors(hBondIter,1)+idxShiftGromacs);
    
    donorAtomNameH  = pdb.atomName{hBondNdxData.donors(hBondIter,idxDonorH)+idxShiftGromacs};
    donorAtomNameHeavy = pdb.atomName{hBondNdxData.donors(hBondIter,idxDonorHeavy)+idxShiftGromacs};
    acceptorAtomName = pdb.atomName{hBondNdxData.acceptors(hBondIter,1)+idxShiftGromacs};
    
    donorResidueNrH  = pdb.residueNumber(hBondNdxData.donors(hBondIter,idxDonorH)+idxShiftGromacs);
    donorResidueNrHeavy = pdb.residueNumber(hBondNdxData.donors(hBondIter,idxDonorHeavy)+idxShiftGromacs);
    acceptorResiudeNr = pdb.residueNumber(hBondNdxData.acceptors(hBondIter,1)+idxShiftGromacs);
    
    donorResidueNameH  = pdb.residueName{hBondNdxData.donors(hBondIter,idxDonorH)+idxShiftGromacs};
    donorResidueNameHeavy = pdb.residueName{hBondNdxData.donors(hBondIter,idxDonorHeavy)+idxShiftGromacs};
    acceptorResiudeName = pdb.residueName{hBondNdxData.acceptors(hBondIter,1)+idxShiftGromacs};
    
    %     hBondDonorCoords(end+1,:) = pdb.coords(hBondNdxData.donors(hBondIter,idxDonorH)+idxShiftGromacs, :);
    %     hBondAcceptorCoords(end+1,:) = pdb.coords(hBondNdxData.acceptors(hBondIter,1)+idxShiftGromacs, :);
    
    pdbIndexDonor    = hBondNdxData.donors(hBondIter,idxDonorH)+idxShiftGromacs; % we need the gromacs shift for the coords
    pdbIndexAcceptor = hBondNdxData.acceptors(hBondIter,1)+idxShiftGromacs;
    
    %     if verbose
    %         fprintf ('hbond nr %3.0i (present %2.1f%% of the time):\n', hBondIter, hbBondPresence(hBondIter)*100);
    %         fprintf ('\t donor   : atom %4.0i (%s) which is part of %s%1.0f (heavy atom: %4.0i (%s) which is part of %s%1.0f)\n', donorAtomNrH, donorAtomNameH, donorResidueNameH, donorResidueNrH, donorAtomNrHeavy, donorAtomNameHeavy, donorResidueNameHeavy, donorResidueNrHeavy);
    %         fprintf ('\t acceptor: atom %4.0i (%s) which is part of %s%1.0f\n', acceptorAtomNr, acceptorAtomName, acceptorResiudeName, acceptorResiudeNr);
    %     end
    
    % do the checks even if not verbose
    if donorResidueNrH ~= donorResidueNrHeavy
        error ('The H of the donor and the heavy atom of the donor must be in the same residue!');
    end
    foundH = strfind (donorAtomNameH, 'H');
    if isempty (foundH)
        error ('The donor hydrogen must be of type H (so the string must contain a "H" somewhere) but found "%s"', donorAtomNameH);
    end
    if strcmp(acceptorAtomName, 'H')
        error ('The acceptor atom must NOT be of type H (the string must NOT be "H") but found "%s"', donorAtomNameH);
    end
    
    currHBondPresence = hbBondPresence(hBondIter); % the radius of the cylinder is proportinal to the presence of h-bonds
    
    [pdbIndexAcceptorCa] = findMyCAlpha (pdbIndexAcceptor, pdb);
    [pdbIndexDonorCa] = findMyCAlpha (pdbIndexDonor, pdb);
    
    d_mtx =  [pdb.coords(pdbIndexDonorCa, 1), pdb.coords(pdbIndexAcceptorCa, 1); pdb.coords(pdbIndexDonorCa, 2), pdb.coords(pdbIndexAcceptorCa, 2); pdb.coords(pdbIndexDonorCa, 3), pdb.coords(pdbIndexAcceptorCa, 3)];
    dCA = dist (d_mtx); dCA = dCA(1,2);
    
    d_mtx =  [pdb.coords(pdbIndexDonor, 1), pdb.coords(pdbIndexAcceptor, 1); pdb.coords(pdbIndexDonor, 2), pdb.coords(pdbIndexAcceptor, 2); pdb.coords(pdbIndexDonor, 3), pdb.coords(pdbIndexAcceptor, 3)];
    dAtom = dist (d_mtx); dAtom = dAtom(1,2);
    
    if (dCA > 31 || dAtom > 31) && (hbBondPresence(hBondIter) > 0.1) % if they are further apart than 30 and this happens more than 10% of the time we might have a problem (not neccesarily)
        fprintf ('hbond nr %3.0i (present %2.1f%% of the time):\n', hBondIter, hbBondPresence(hBondIter)*100);
        fprintf ('\t donor   : atom %4.0i (%s) which is part of %s%1.0f (heavy atom: %4.0i (%s) which is part of %s%1.0f)\n', donorAtomNrH, donorAtomNameH, donorResidueNameH, donorResidueNrH, donorAtomNrHeavy, donorAtomNameHeavy, donorResidueNameHeavy, donorResidueNrHeavy);
        fprintf ('\t acceptor: atom %4.0i (%s) which is part of %s%1.0f\n', acceptorAtomNr, acceptorAtomName, acceptorResiudeName, acceptorResiudeNr);
        warning ('h-bond more than 30 A apart (CA/CA %1.2f A; donor/acceptor %1.2f)', dCA, dAtom);
    end
    
    
    acceptorResidue = pdb.residueNumber(pdbIndexAcceptorCa); %this works because we have previously created a consecutive numbering of the AAs
    donorResidue = pdb.residueNumber(pdbIndexDonorCa);
    
    
    if acceptorResidue < 1 || donorResidue < 1
        fprintf ('warning: biologist twat has used an index < 1 for the residue ');
        fprintf('details: h-bond from {%1.3f %1.3f %1.3f} to {%1.3f %1.3f %1.3f} presence %1.2f; # residues %s%i(%s)-%s%i(%s) # DONOR atomNr=%i (%s) of %s%1.0f (heavy atomNr=%i (%s) of %s%1.0f # ACCEPTOR atomNR=%i (%s) of %s%1.0f \n',  ...
            pdb.coords(pdbIndexDonorCa, 1), pdb.coords(pdbIndexDonorCa, 2), pdb.coords(pdbIndexDonorCa, 3), ...
            pdb.coords(pdbIndexAcceptorCa, 1), pdb.coords(pdbIndexAcceptorCa, 2), pdb.coords(pdbIndexAcceptorCa, 3), currHBondPresence, ...
            pdb.residueName{pdbIndexDonorCa}, pdb.residueNumber(pdbIndexDonorCa), pdb.chainIdentifier{pdbIndexDonorCa}, pdb.residueName{pdbIndexAcceptorCa}, pdb.residueNumber(pdbIndexAcceptorCa), pdb.chainIdentifier{pdbIndexAcceptor}, ...
            donorAtomNrH, donorAtomNameH, donorResidueNameH, donorResidueNrH, donorAtomNrHeavy, donorAtomNameHeavy, donorResidueNameHeavy, donorResidueNrHeavy, ...
            acceptorAtomNr, acceptorAtomName, acceptorResiudeName, acceptorResiudeNr ...
            );
        
    else
        mtx(acceptorResidue, donorResidue) = mtx(acceptorResidue, donorResidue) + currHBondPresence;
    end
    
    % fprintf('%1.0f\n', acceptorResidue);
    
    %     end
    
end

a = 1;

for xIter = 1:size(mtx,1)
    for yIter = 1:size(mtx,2)
        if yIter > xIter
            % an hbond from r45 to r109 is the same as an hbond from r109 to r45
            mtx(yIter, xIter) = mtx(yIter, xIter) + mtx(xIter, yIter);
            mtx(xIter, yIter) = 0;
        end
        if xIter == yIter % self h-bonds are not interesting for us here
            mtx(xIter, yIter) = 0;
        end
    end
end

% figure;
% imagesc(mtx);
% myColorMap = jet;
% myColorMap(1,:) = [1 1 1];
% % caxis ([0, 2]);
% colormap(myColorMap);
% colorbar;

a = 1;



function [pdb] = changeResidueIndicesToConsecutiveNumbering (pdb, pdbCA)

currentNumber = 0; % start invalid but first iteration is an increase
prevResNr = -1;
currResNr = -1;
for atomsIter = 1:size(pdb.atomNumber)
    prevResNr = currResNr;
    currResNr = pdb.residueNumber(atomsIter);
    if prevResNr ~= currResNr
        currentNumber = currentNumber + 1;
    end
    pdb.residueNumber (atomsIter) = currentNumber;
end

if length(pdbCA.coords) ~= currentNumber
    error ('numbered %1.0f residues but the CA only model has only %1.0f CAs - there is something wrong', currentNumber, length(pdbCA.coords));
end

a = 1;




