function [hBondMatrix, hBondNdxData, hBondNum, pdb, allAtomMtxGroup1, allAtomMtxGroup2, allResMtxGroup1, allResMtxGroup2] = parseHBond_allOutput (fileName_hbm, fileName_hbn, fileName_num, fileName_pdb, verbose)
% this function parses and analysis all information available of the g_hbonds output:
% - fileName_hbm: the matrix
% - fileName_hbn: the index information for the y-axis of the matrix
% - fileName_num: number of hbonds over time (used for internal validation
% of this function only: should be the same as the sums of the matrix)
% - fileName_pdb: the atoms information for the indx information of hbn
%
% alternatively the already parsed data can be provided.
%
% Make sure that both groups are (1) consecutive (do not use things like "chain A and resid 1-180 or chain B") and (2) non-overlapping or identical (see gromacs manual)!
%
% example for usage:
% myBaseName = '/run/media/knapp/Seagate Expansion Drive/1mi5_172_100ns_sims/1A/';
% parseHBond_allOutput ([myBaseName, '1A.hbondMtx.TCRvsPeptide.xvg.xpm'], [myBaseName, '1A.hbondNdx.TCRvsPeptide.ndx'], [myBaseName, '1A.hbondNum.TCRvsPeptide.xvg'], [myBaseName, '1A.firstFrame.pdb'], true);
% parseHBond_allOutput ([myBaseName, '1A.hbondMtx.MHCvsPeptide.xvg.xpm'], [myBaseName, '1A.hbondNdx.MHCvsPeptide.ndx'], [myBaseName, '1A.hbondNum.MHCvsPeptide.xvg'], [myBaseName, '1A.firstFrame.pdb'], true);
%
%
% B. Knapp 2015-08-21


if ischar(fileName_hbm) && ischar(fileName_hbn) && ischar(fileName_num) && ischar(fileName_pdb)
    % fprintf ('parsing index file ... ');
    hBondNdxData = parseHBond_hbn (fileName_hbn);
    % fprintf ('done. parsing matrix file ... ');
    hBondMatrix = parseHBond_hbm (fileName_hbm, false);
    % fprintf ('done. parsing numbers file ... ');
    hBondNum = parseXvgFile (fileName_num);
    % fprintf ('done. parsing pdb file ... ');
    pdb = parsePdb(fileName_pdb);
    % fprintf ('done.\n');
else
    % one can also provide the inputs in already parsed format (in this case
    % the input is not of type string). this is usefull in mass evaluations
    % where parsing is done only once but one might still be interested in the
    % matrices produced in this function.
    hBondNdxData = fileName_hbn;
    hBondMatrix = fileName_hbm;
    hBondNum = fileName_num;
    pdb = fileName_pdb;
end

% check if the number of different h-bond corresponds between index and matrix file
nrHBondNdx = size(hBondNdxData.acceptors,1);
nrHBondMtx = size(hBondMatrix.mtx,1);
if nrHBondNdx ~= nrHBondMtx
    error ('Number of hydrogen bonds does not match (index file: %i and matrix file: %i). Are you parsing the matching files ("%s" and "%s")?', nrHBondNdx, nrHBondMtx, fileName_hbn, fileName_hbm);
end

% check if the number of h-bonds per timestep is identical between number and matrix file
diff = sum(abs(hBondNum.vals(:,2)' - sum(hBondMatrix.mtx,1)));
doubleHBonds = sum(abs(hBondNum.vals(:,2)'))/2; % if we use overlapping groups then the matrix will have exactly 2* the number of h-bonds as the xvg file
maxDiff = 5; % in very rare cases gromacs doesnt do this the same way (maybe they use smaller and smaller equal or something but (very) few dots can be missing
if (diff > 1 || diff < -1) && (diff ~= doubleHBonds)
    warning ('Found a total of %i differences between the number file and the sum per timeframe matrix file. Are you parsing the matching files ("%s" and "%s")?', diff, fileName_num, fileName_hbm);
end
if (diff > maxDiff || diff < -maxDiff) && (diff ~= doubleHBonds)
    error ('Found a total of %i differences between the number file and the sum per timeframe matrix file. Are you parsing the matching files ("%s" and "%s")?', diff, fileName_num, fileName_hbm);
end

allAtomMtxGroup1 = createWholeMatrix (hBondMatrix, hBondNdxData, 1, verbose);
allAtomMtxGroup2 = createWholeMatrix (hBondMatrix, hBondNdxData, 2, verbose);

allResMtxGroup1 = summarizeMtxPerResiude (allAtomMtxGroup1, hBondMatrix, hBondNdxData, 1, pdb, verbose);
allResMtxGroup2 = summarizeMtxPerResiude (allAtomMtxGroup2, hBondMatrix, hBondNdxData, 2, pdb, verbose);

% useCAsForVmd = true;
% writeVmdFileForHBondPlots (hBondNdxData, hBondMatrix, pdb, verbose, useCAsForVmd);

% if verbose
%     figure;
%     plotXvgFile(hBondNum);
%     figure;
%     plotXpm_from_hBondMatrix(hBondMatrix)
%     figure;
%     plotPdb(pdb, 'coloringMethod', 'chain', 'drawingMethod', 'lines', 'plotOnlyCA', true);
%     hold on;
%     for hBondIter = 1:size(hBondAcceptorCoords,1) % please note that these hbonds are all which are ever present over time so not all of them make sence to be drawn on all frames ...
%         plot3([hBondDonorCoords(hBondIter, 1), hBondAcceptorCoords(hBondIter, 1)], [hBondDonorCoords(hBondIter, 2), hBondAcceptorCoords(hBondIter, 2)], [hBondDonorCoords(hBondIter, 3), hBondAcceptorCoords(hBondIter, 3)], '--k');
%     end
% end

% allAtomMtxGroup1 = [];
% allAtomMtxGroup2 = [];

function [resMtx] = summarizeMtxPerResiude (atomMtx, hBondMatrix, hBondNdxData, groupId, pdb, verbose)
% this function summarizes the atom matrix created by the function
% "createWholeMatrix" per residue. i.e. the peptide has only 9 datapoints
% (one per residue instead of one per atom)

if groupId == 1
    offSetAtom = hBondNdxData.group1AtomNrs(1)-1; % -1 to set it before the first pos (offset + iter = frist pos)
    lastAtom = hBondNdxData.group1AtomNrs(end);
    myName = hBondNdxData.group1Name;
    myPartner = hBondNdxData.group2Name;
elseif groupId == 2
    offSetAtom = hBondNdxData.group2AtomNrs(1)-1;
    lastAtom = hBondNdxData.group2AtomNrs(end);
    myName = hBondNdxData.group2Name;
    myPartner = hBondNdxData.group1Name;
else
    error ('groupId must be 1 or 2');
end

prevResIdx = -99;

resMtx = [];
listOfResIds = [];
chainOfResIds = {};

for atomIterRelative = 1:size(atomMtx,1)
    atomIterAbsolute = offSetAtom+atomIterRelative;
    resIdx = pdb.residueNumber(atomIterAbsolute);
    
    % fprintf ('atomId = %i, resid = %i, resname = %s, chain = %s\n', pdb.atomNumber(atomIterAbsolute), pdb.residueNumber(atomIterAbsolute), pdb.residueName{atomIterAbsolute}, pdb.chainIdentifier{atomIterAbsolute});
    
    if prevResIdx ~= resIdx % new residue found
        posToWrite = size(resMtx, 1)+1;
        prevResIdx = resIdx;
        listOfResIds(end+1) = resIdx;
        chainOfResIds{end+1} = pdb.chainIdentifier{atomIterAbsolute};
    end
    if size(resMtx,1) < posToWrite % new line
        resMtx(posToWrite,:) = atomMtx(atomIterRelative, :);
        
    else % add up
        resMtx(posToWrite,:) = resMtx(posToWrite,:) + atomMtx(atomIterRelative, :);
    end
end

sumHbondsAtoms = sum(sum(atomMtx));
sumHbondsResidues = sum(sum(resMtx));

if sumHbondsAtoms~=sumHbondsResidues
    error ('The number of hbonds in the atom matrix (found %i) must be equal to the number of hbonds in the residue matrix (found %i). There is a problem with the implementation ... ', sumHbondsAtoms, sumHbondsResidues);
end

if verbose
    figure;
    %colormap([1 1 1; 0 0 0]);
    imagesc(flipud(resMtx)); % for some reason it is standing on its head ...
    set(gca,'YDir','normal');
    colorbar;
    %caxis([0, 1]);
    xlabel(hBondMatrix.headerInfo.xLabel);
    ylabel('Residue index within group');
    stretchFactorX = 100;
    set(gca, 'XTick', hBondMatrix.headerInfo.xAxis(1):stretchFactorX:hBondMatrix.headerInfo.xAxis(end));
    set(gca, 'XTickLabel', hBondMatrix.headerInfo.xAxis(1:stretchFactorX:end));
    myName(myName=='_') = '&';
    myPartner(myPartner=='_') = '&';
    title(['hbonds of ', myName, ' (partner ', myPartner, ')']);
    myColorMap = jet;
    myColorMap(1,:) = [1 1 1]; % zero values white
    colormap(myColorMap);
    
    isHbond = sum(resMtx,2);
    vmdCmd = '';
    for resIter = 1:size(listOfResIds, 2)
        if isHbond(resIter)
            vmdCmd = sprintf('%s or (resid %i and chain %s)', vmdCmd, listOfResIds(resIter), chainOfResIds{resIter});
        end
    end
    fprintf('\nvisualize in VMD via:\n%s\n', vmdCmd(4:end));
    
end


function [allAtomsMtxGroup] = createWholeMatrix (hBondMatrix, hBondNdxData, groupId, verbose)
% in the original matrix only the atoms with hydrogen bonds are contained.
% therefore the matrices are not compareable between mutants. here we copy
% the "lines" of hydrogens is a big (mostly zero) matrix.

if groupId == 1 % check if we do it for group 1 or 2 of the h-bonds
    nrAtomsGroup = size(hBondNdxData.group1AtomNrs,2);
    myName = hBondNdxData.group1Name;
elseif groupId == 2
    nrAtomsGroup = size(hBondNdxData.group2AtomNrs,2);
    myName = hBondNdxData.group2Name;
else
    error ('groupId must be 1 or 2');
end

nrHBondMtx = size(hBondMatrix.mtx,1); % preallocate matrix
nrOfTimeSteps = size(hBondMatrix.mtx,2);

allAtomsMtxGroup = zeros (nrAtomsGroup, nrOfTimeSteps);

if groupId == 1 % calculate offset (we do not want a huge zero matrix at the beginning)
    minIdx = min(hBondNdxData.group1AtomNrs);
else
    minIdx = min(hBondNdxData.group2AtomNrs);
end

for mtxCopyIter = 1:nrHBondMtx
    idx = hBondNdxData.donors(mtxCopyIter,1)-minIdx+1; % check the donor indexes first
    if idx > 0 && idx < nrAtomsGroup % if the index is valid our group has this atom as donor
        % if this crahes here with index out of bound then make sure that
        % both groups are (1) consecutive (do not do things like chain A and resid 1-180 or chain B) and (2) non-overlapping or identical!
        allAtomsMtxGroup(idx , :) = allAtomsMtxGroup(idx , :) + hBondMatrix.mtx(mtxCopyIter,:);
    else % if the index is invalid our group has the index as acceptor
        idx = hBondNdxData.acceptors(mtxCopyIter)-minIdx+1;
        allAtomsMtxGroup(idx , :) = allAtomsMtxGroup(idx , :) + hBondMatrix.mtx(mtxCopyIter,:);
    end
end

sumHbondsOrigMatrix = sum(sum(hBondMatrix.mtx)); % if those number do not match we have made a mistake while copying them in ...
sumHBondsWholeMatrix = sum(sum(allAtomsMtxGroup));
if  sumHbondsOrigMatrix~=sumHBondsWholeMatrix
    error ('there is a problem with copying the h-bonds (%i hbonds in the original matrix vs % i hbonds in the whole one).', sumHbondsOrigMatrix, sumHBondsWholeMatrix);
end

if verbose
    figure;
    colormap([1 1 1; 0 0 0]);
    imagesc(flipud(allAtomsMtxGroup)); % for some reason it is standing on its head ...
    set(gca,'YDir','normal');
    colorbar;
    caxis([0, 1]);
    xlabel(hBondMatrix.headerInfo.xLabel);
    ylabel(sprintf('Atom index within group (offset to all atoms =%i)',minIdx-1));
    stretchFactorX = 100;
    set(gca, 'XTick', hBondMatrix.headerInfo.xAxis(1):stretchFactorX:hBondMatrix.headerInfo.xAxis(end));
    set(gca, 'XTickLabel', hBondMatrix.headerInfo.xAxis(1:stretchFactorX:end));
    
    myName(myName=='_') = ' ';
    title(myName);
end


